"""DROID circuits module.

These circuits are auto-generated from circuits.json for droid firmware version
'{{ droid_version }}'.


.. include:: README.md
"""

__docformat__ = "google"


{% for category, collection in categories.items() %}
from .{{ category }} import (
    {% for circuit in collection.circuits %}
    {{ circuit.name|capitalize }},
    {% endfor %}
)
{% endfor %}


__all__ = (
{% for category, collection in categories.items() %}
    {% for circuit in collection.circuits %}
    "{{ circuit.name|capitalize }}",
    {% endfor %}
{% endfor %}
)
